#pragma once

#include "../base/EntityBase.h"

#include "EntitySnapshot.h"
#include "detail/CopyWithoutData.h"


namespace armarx::armem::wm
{
    /**
     * @brief An entity over a period of time.
     *
     * An entity should be a physical thing or abstract concept existing
     * (and potentially evolving) over some time.
     *
     * Examples are:
     * - objects (the green box)
     * - agents (robot, human)
     * - locations (frige, sink)
     * - grasp affordances (general, or for a specific object)
     * - images
     * - point clouds
     * - other sensory values
     *
     * At each point in time (`EntitySnapshot`), the entity can have a
     * (potentially variable) number of instances (`EntityInstance`),
     * each containing a single `AronData` object of a specific `AronType`.
     */
    class Entity :
        public base::EntityBase<EntitySnapshot, Entity>,
        public detail::CopyWithoutData<Entity>
    {
        using Base = base::EntityBase<EntitySnapshot, Entity>;

    public:

        using Base::EntityBase;

        Entity(const Entity& other) = default;
        Entity(Entity&& other) = default;
        Entity& operator=(const Entity& other) = default;
        Entity& operator=(Entity&& other) = default;

        virtual ~Entity() override = default;


    protected:

        virtual void _copySelfWithoutData(Entity& other) const override;

    };

}
