#include "aron_conversions.h"

#include <algorithm>
#include <cstdint>
#include <iterator>

#include <RobotAPI/interface/units/LaserScannerUnit.h>
#include <RobotAPI/libraries/armem_robot_mapping/aron/LaserScan.aron.generated.h>
#include <RobotAPI/libraries/aron/converter/common/Converter.h>
#include <RobotAPI/libraries/aron/core/navigator/data/complex/NDArray.h>

#include "types.h"


namespace armarx
{

    /************ fromAron ************/

    armem::Time timeFromAron(const int64_t timestamp)
    {
        return armem::Time::microSeconds(timestamp);
    }

    SensorHeader fromAron(const arondto::SensorHeader& aronSensorHeader)
    {

        return {.agent = aronSensorHeader.agent,
                .frame = aronSensorHeader.frame,
                .timestamp = timeFromAron(aronSensorHeader.timestamp)};
    }

    void fromAron(const arondto::LaserScanStamped& aronLaserScan,
                  LaserScanStamped& laserScan)
    {
        laserScan.header = fromAron(aronLaserScan.header);
        // laserScan.data = fromAron(aronLaserScan.data);
    }

    void fromAron(const arondto::LaserScanStamped& aronLaserScan, LaserScan& laserScan,
                  std::int64_t& timestamp, std::string& frame,
                  std::string& agentName)
    {
        const auto header = fromAron(aronLaserScan.header);

        // laserScan = fromAron(aronLaserScan.data);

        timestamp = header.timestamp.toMicroSeconds();
        frame = header.frame;
        agentName = header.agent;
    }


    /************ toAron ************/


    // auto toAron(const LaserScan& laserScan, aron::LaserScan& aronLaserScan)
    // {
    //     aronLaserScan.scan = toAron(laserScan);
    // }

    int64_t toAron(const armem::Time& timestamp)
    {
        return timestamp.toMicroSeconds();
    }

    arondto::SensorHeader toAron(const SensorHeader& sensorHeader)
    {
        arondto::SensorHeader aronSensorHeader;

        aronSensorHeader.agent = sensorHeader.agent;
        aronSensorHeader.frame = sensorHeader.frame;
        aronSensorHeader.timestamp = toAron(sensorHeader.timestamp);

        return aronSensorHeader;
    }

    void toAron(const LaserScanStamped& laserScanStamped,
                arondto::LaserScanStamped& aronLaserScanStamped)
    {
        aronLaserScanStamped.header = toAron(laserScanStamped.header);
        // toAron(laserScanStamped.data, aronLaserScanStamped.data);
    }

    void toAron(const LaserScan& laserScan,
                const armem::Time& timestamp,
                const std::string& frame,
                const std::string& agentName,
                arondto::LaserScanStamped& aronLaserScanStamped)
    {
        const SensorHeader header
        {
            .agent = agentName, .frame = frame, .timestamp = timestamp};

        const LaserScanStamped laserScanStamped{.header = header, .data = laserScan};

        toAron(laserScanStamped, aronLaserScanStamped);
    }

} // namespace armarx