/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <unordered_map>

// Base Class
#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/SerializerFactory.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/type/AllNavigators.h>
#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/cpp/serializer/AllSerializers.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

namespace armarx::aron::cppcodegenerator
{
    class SerializerFactory;
    typedef std::shared_ptr<SerializerFactory> SerializerFactoryPtr;

    class SerializerFactory :
        virtual public codegenerator::SerializerFactory<typenavigator::NavigatorPtr, SerializerPtr>
    {
    public:
        SerializerFactory() = default;
        virtual SerializerPtr create(const typenavigator::NavigatorPtr&, const Path&) const override;
        virtual SerializerPtr createSpecific(const typenavigator::NavigatorPtr&, const Path&) const override;

    };

    // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class upperType##SerializerFactory : \
        virtual public SerializerFactory \
    { \
    public: \
        upperType##SerializerFactory() = default; \
        virtual SerializerPtr createSpecific(const typenavigator::NavigatorPtr&, const Path&) const override; \
    };

    HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
}
