/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/core/io/dataIO/writer/WriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/navigator/data/AllNavigators.h>

namespace armarx::aron::dataIO::writer
{
    class NavigatorWriterToken;
    typedef std::shared_ptr<NavigatorWriterToken> NavigatorWriterTokenPtr;

    class NavigatorWriterToken :
        virtual public WriterTokenInterface<datanavigator::NavigatorPtr>
    {
    public:
        using PointerType = NavigatorWriterTokenPtr;

    public:
        // constructor
        NavigatorWriterToken() = delete;
        NavigatorWriterToken(const data::Descriptor desc, const datanavigator::NavigatorPtr& d)
        {
            descriptor = desc;
            element = d;
        }

        // public member functions
        virtual void addElement(const datanavigator::NavigatorPtr& n) override
        {
            auto desc = this->getDescriptor();
            switch (desc)
            {
                case data::Descriptor::eDict:
                {
                    datanavigator::DictNavigatorPtr casted = datanavigator::DictNavigator::DynamicCastAndCheck(element);
                    casted->addElement(currentKey, n);
                    break;
                }
                case data::Descriptor::eList:
                {
                    datanavigator::ListNavigatorPtr casted = datanavigator::ListNavigator::DynamicCastAndCheck(element);
                    casted->addElement(n);
                    currentIndex++;
                    break;
                }
                default:
                    throw error::DescriptorNotValidException("NavigatorWriterToken", "addElement", "Could not resove a type of a navigator. Allowed are only containers. The path was: " + element->pathToString(), desc);
            }
        }
    };
}
