/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "List.h"

namespace armarx::aron::typenavigator
{
    // constructors
    ListNavigator::ListNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eList, path),
        Navigator(type::Descriptor::eList, path),
        type(new type::AronList())
    {
    }

    ListNavigator::ListNavigator(const type::AronListPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eList, path),
        Navigator(type::Descriptor::eList, path),
        type(o)
    {
        CheckAronPtrForNull("ListNavigator", "ListNavigator", getPath(), o);
        acceptedType = FromAronType(o->acceptedType);
    }

    // Member functions
    NavigatorPtr ListNavigator::getAcceptedType() const
    {
        return acceptedType;
    }

    void ListNavigator::setAcceptedType(const NavigatorPtr& a)
    {
        CheckTypeNavigatorPtrForNull("ListNavigator", "setAcceptedType", a);
        type->acceptedType = a->getResult();
        acceptedType = a;
    }

    // static methods
    ListNavigatorPtr ListNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<ListNavigator>(n);
    }

    ListNavigatorPtr ListNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("ListNavigator", "DynamicCast[Before]", n);
        ListNavigatorPtr casted = ListNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("ListNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    type::AronListPtr ListNavigator::toAronListPtr() const
    {
        CheckAronPtrForNull("ListNavigator", "getCastedResult", getPath(), type->acceptedType);
        return type;
    }

    // virtual implementations
    std::vector<NavigatorPtr> ListNavigator::getChildren() const
    {
        return {acceptedType};
    }

    size_t ListNavigator::childrenSize() const
    {
        return 1;
    }

    type::AronTypePtr ListNavigator::getResult() const
    {
        return toAronListPtr();
    }

    std::string ListNavigator::getName() const
    {
        return "AronListType<" + acceptedType->getName() + ">";
    }
}
