/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "IntEnum.h"

namespace armarx::aron::typenavigator
{
    // constructors
    IntEnumNavigator::IntEnumNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eIntEnum, path),
        Navigator(type::Descriptor::eIntEnum, path),
        type(new type::AronIntEnum())
    {
    }

    IntEnumNavigator::IntEnumNavigator(const type::AronIntEnumPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eIntEnum, path),
        Navigator(type::Descriptor::eIntEnum, path),
        type(o)
    {
        CheckAronPtrForNull("AronEnumTypeNavigator", "AronEnumTypeNavigator", getPath(), o);
    }

    std::map<std::string, int> IntEnumNavigator::getAcceptedValues() const
    {
        return type->acceptedValues;
    }

    std::string IntEnumNavigator::getValueName(int i) const
    {
        for (const auto& [k, v] : type->acceptedValues)
        {
            if (v == i)
            {
                return k;
            }
        }
        throw error::AronException("IntEnumNavigator", "getValueName", "Enum could not be resolved. Input was: " + std::to_string(i), getPath());
    }

    int IntEnumNavigator::getValue(const std::string& s) const
    {
        return type->acceptedValues[s];
    }

    std::string IntEnumNavigator::getEnumName() const
    {
        return type->enumName;
    }

    void IntEnumNavigator::setEnumName(const std::string& s)
    {
        type->enumName = s;
    }

    void IntEnumNavigator::addAcceptedValue(const std::string& s, int i)
    {
        type->acceptedValues[s] = i;
    }

    type::AronIntEnumPtr IntEnumNavigator::toAronIntEnumPtr() const
    {
        return type;
    }

    // static methods
    IntEnumNavigatorPtr IntEnumNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<IntEnumNavigator>(n);
    }

    IntEnumNavigatorPtr IntEnumNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("AronEnumTypeNavigator", "DynamicCast[Before]", n);
        IntEnumNavigatorPtr casted = IntEnumNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("AronEnumTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> IntEnumNavigator::getChildren() const
    {
        return {};
    }

    size_t IntEnumNavigator::childrenSize() const
    {
        return 0;
    }

    type::AronTypePtr IntEnumNavigator::getResult() const
    {
        return toAronIntEnumPtr();
    }

    std::string IntEnumNavigator::getName() const
    {
        return "AronIntEnumType";
    }
}

