/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "EigenQuaternion.h"

namespace armarx::aron::typenavigator
{
    // constructors
    EigenQuaternionNavigator::EigenQuaternionNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eEigenQuaternion, path),
        Navigator(type::Descriptor::eEigenQuaternion, path),
        type(new type::AronEigenQuaternion(""))
    {
    }

    EigenQuaternionNavigator::EigenQuaternionNavigator(const type::AronEigenQuaternionPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eEigenQuaternion, path),
        Navigator(type::Descriptor::eEigenQuaternion, path),
        type(o)
    {
        CheckAronPtrForNull("AronEigenQuaternionTypeNavigator", "AronEigenQuaternionTypeNavigator", getPath(), o);
    }

    bool EigenQuaternionNavigator::checkTypename(const std::string& s) const
    {
        if (s.empty())
        {
            throw error::AronException("AronEigenQuaternionTypeNavigator", "checkTypename", "The typename is empty.", getPath());
        }

        if (!std::any_of(ACCEPTED_TYPES.begin(), ACCEPTED_TYPES.end(), [s](const auto & el)
    {
        return s == el.first || std::find(el.second.begin(), el.second.end(), s) != el.second.end();
        }))
        {
            throw error::AronException("AronEigenQuaternionTypeNavigator", "checkTypename", "The typename is wrong. Got " + s, getPath());
        }
        return true;
    }

    type::AronEigenQuaternionPtr EigenQuaternionNavigator::toAronEigenQuaternionPtr() const
    {
        checkTypename(type->typeName);
        return type;
    }

    std::string EigenQuaternionNavigator::getTypename() const
    {
        return type->typeName;
    }

    std::vector<int> EigenQuaternionNavigator::getDimensions() const
    {
        return ACCEPTED_DIMENSION;
    }

    void EigenQuaternionNavigator::setTypename(const std::string& u)
    {
        checkTypename(u);
        type->typeName = u;
    }

    // static methods
    EigenQuaternionNavigatorPtr EigenQuaternionNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<EigenQuaternionNavigator>(n);
    }

    EigenQuaternionNavigatorPtr EigenQuaternionNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("AronEigenQuaternionTypeNavigator", "DynamicCast[Before]", n);
        EigenQuaternionNavigatorPtr casted = EigenQuaternionNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("AronEigenQuaternionTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> EigenQuaternionNavigator::getChildren() const
    {
        return {};
    }

    size_t EigenQuaternionNavigator::childrenSize() const
    {
        return 0;
    }

    type::AronTypePtr EigenQuaternionNavigator::getResult() const
    {
        return toAronEigenQuaternionPtr();
    }

    std::string EigenQuaternionNavigator::getName() const
    {
        return "AronEigenQuaternionType<" + simox::alg::to_string(ACCEPTED_DIMENSION, ", ") + ", " + type->typeName + ">";
    }
}

